### R code from vignette source '02.Rnw'

###################################################
### code chunk number 1: c0
###################################################
suppressPackageStartupMessages(require(SOAR))
options(width=75)
graphics.off()
pdf(file="Fig/01_All.pdf", onefile=TRUE)
if(length(ls() > 0)) Store(ls())
options(continue="  ",
        SweaveHooks = list(fig = function() par(las = 1,
        family = "mono")))
## Sys.setenv(R_LOCAL_CACHE=".R_Store")
Attach()


###################################################
### code chunk number 2: c01
###################################################
data(janka)
library(SOAR)
Store(janka)   ## for convenience
plot(Hardness ~ Density, janka, pch=20, col="red")


###################################################
### code chunk number 3: c02
###################################################
m1 <- lm(Hardness ~ Density, janka)
m2 <- update(m1, . ~ . + I(Density^2))
m3 <- update(m2, . ~ . + I(Density^3))
round(summary(m3)$coef, 4)


###################################################
### code chunk number 4: c03
###################################################
m1a <- lm(Hardness ~ I(Density - 50), janka)
m2a <- update(m1a, . ~ . + I((Density - 50)^2))
m3a <- update(m2a, . ~ . + I((Density - 50)^3))
round(summary(m3a)$coef, 4)


###################################################
### code chunk number 5: c04
###################################################
pJanka <- with(janka, data.frame(Density = seq(-5, 75)))
pJanka <- within(pJanka, {
  pM1 <- predict(m1a, pJanka, type = "response")
  pM2 <- predict(m2a, pJanka, type = "resp")
  pM3 <- predict(m3a, pJanka, type = "resp")
})
r0 <- with(pJanka, with(janka, range(Hardness, pM1, pM2, pM3)))

plot(pM1 ~ Density, pJanka, ylim = r0, ylab = "Density",
     type = "l", col="darkgreen")
lines(pM2 ~ Density, pJanka, col = "blue")
lines(pM3 ~ Density, pJanka, col = "gold4")
points(Hardness ~ Density, janka, pch = 20, col = grey(0.6))
abline(v = c(0, 50), col = grey(0.5), lty = "dashed")
axis(1, at = c(0, 50), font = 2, col.axis = "red")
legend("bottomright", c("linear", "quadratic", "cubic"),
       lty = "solid", col = c("darkgreen", "blue", "gold4"),
       text.col = c("darkgreen", "blue", "gold4"), bty="n")


###################################################
### code chunk number 6: c05
###################################################
pJanka <- with(janka, data.frame(Density = seq(-5, 75)))
pJanka <- within(pJanka, {
  pM1 <- predict(m1a, pJanka, type = "response")
  pM2 <- predict(m2a, pJanka, type = "resp")
  pM3 <- predict(m3a, pJanka, type = "resp")
})
r0 <- with(pJanka, with(janka, range(Hardness, pM1, pM2, pM3)))

plot(pM1 ~ Density, pJanka, ylim = r0, ylab = "Density",
     type = "l", col="darkgreen")
lines(pM2 ~ Density, pJanka, col = "blue")
lines(pM3 ~ Density, pJanka, col = "gold4")
points(Hardness ~ Density, janka, pch = 20, col = grey(0.6))
abline(v = c(0, 50), col = grey(0.5), lty = "dashed")
axis(1, at = c(0, 50), font = 2, col.axis = "red")
legend("bottomright", c("linear", "quadratic", "cubic"),
       lty = "solid", col = c("darkgreen", "blue", "gold4"),
       text.col = c("darkgreen", "blue", "gold4"), bty="n")


###################################################
### code chunk number 7: c06
###################################################
layout(matrix(1:4, 2,2, byrow = TRUE))
plot(m2a)


###################################################
### code chunk number 8: c07
###################################################
library(MASS)
layout(matrix(1:2, 1))
boxcox(m1a, lambda = seq( 0.2, 0.75, len=10))
title(main = "straight line model")
boxcox(m2a, lambda = seq(-0.25, 0.6, len=10))
title(main = "quadratic model")


###################################################
### code chunk number 9: c08
###################################################
pdf(file = "Fig/02a.pdf", height=4.5, width = 8)
par(las=1)
library(MASS)
layout(matrix(1:2, 1))
boxcox(m1a, lambda = seq( 0.2, 0.75, len=10))
title(main = "straight line model")
boxcox(m2a, lambda = seq(-0.25, 0.6, len=10))
title(main = "quadratic model")
dev.off()


###################################################
### code chunk number 10: c09
###################################################
mSL <- update(m1a, sqrt(.) ~ .)
mQD <- update(m2a, log(.) ~ .)
r1 <- resid(mSL); f1 <- fitted(mSL)
r2 <- resid(mQD); f2 <- fitted(mQD)
layout(matrix(1:2, 1))
plot(f1, r1, xlab = "fitted values", ylab = "residuals",
     main = expression(sqrt(italic(H)) == beta[0] +
         beta[1]*italic(D)))
abline(h=0, col="red", lty="dashed")
plot(f2, r2, xlab = "fitted values", ylab = "residuals",
     main = expression(log~italic(H) == beta[0] +
         beta[1]*italic(D) + beta[2]*italic(D)^2))
abline(h=0, col="red", lty="dashed")


###################################################
### code chunk number 11: c10
###################################################
pdf(file = "Fig/02b.pdf", height=4.5, width = 8)
par(las=1)
mSL <- update(m1a, sqrt(.) ~ .)
mQD <- update(m2a, log(.) ~ .)
r1 <- resid(mSL); f1 <- fitted(mSL)
r2 <- resid(mQD); f2 <- fitted(mQD)
layout(matrix(1:2, 1))
plot(f1, r1, xlab = "fitted values", ylab = "residuals",
     main = expression(sqrt(italic(H)) == beta[0] +
         beta[1]*italic(D)))
abline(h=0, col="red", lty="dashed")
plot(f2, r2, xlab = "fitted values", ylab = "residuals",
     main = expression(log~italic(H) == beta[0] +
         beta[1]*italic(D) + beta[2]*italic(D)^2))
abline(h=0, col="red", lty="dashed")
dev.off()


###################################################
### code chunk number 12: c11
###################################################
library(glm2)
mGLM <- glm2(Hardness ~ I(Density-50),
             family = Gamma(link = make.link("sqrt")),
             data = janka)
mGLM2 <- update(mGLM, .~.+I((Density-50)^2))
round(summary(mGLM2)$coef, 4)


###################################################
### code chunk number 13: c12
###################################################
rs <- resid(mGLM)
fv <- fitted(mGLM)
layout(matrix(1:2, 1))
plot(fv, rs, xlab = "fitted values", ylab = "residuals",
     pch = 20, col=grey(0.5), main = "Variance uniformity")
abline(h = 0, lty = "dashed", col="red")

qqnorm(rs, pch = 20, col = grey(0.5), xlab = "Normal scores",
       ylab = "sorted residuals", main = "Normal Q-Q plot")
qqline(rs, col="red", lty = "dashed")


###################################################
### code chunk number 14: c13
###################################################
pdf(file = "Fig/02c.pdf", height=4.5, width = 8)
par(las=1)
rs <- resid(mGLM)
fv <- fitted(mGLM)
layout(matrix(1:2, 1))
plot(fv, rs, xlab = "fitted values", ylab = "residuals",
     pch = 20, col=grey(0.5), main = "Variance uniformity")
abline(h = 0, lty = "dashed", col="red")

qqnorm(rs, pch = 20, col = grey(0.5), xlab = "Normal scores",
       ylab = "sorted residuals", main = "Normal Q-Q plot")
qqline(rs, col="red", lty = "dashed")
dev.off()


###################################################
### code chunk number 15: c14 (eval = FALSE)
###################################################
## plot(Hardness ~ Density, janka, log = "xy", pch=20, col="red",
##      main = "log-log scale")


###################################################
### code chunk number 16: c15
###################################################
plot(Hardness ~ Density, janka, log = "xy", pch=20, col="red",
     main = "log-log scale")


###################################################
### code chunk number 17: c16
###################################################
LogM <- lm(log(Hardness) ~ poly(log(Density), 3), janka)
round(summary(LogM)$coef, 4)
LogM1 <- update(LogM, . ~ log(Density))
pJanka <- data.frame(Density = 24:70)  ## just covers actual range
prVal <- with(predict(LogM1, pJanka, type = "resp", se.fit = TRUE),
       exp(cbind(fitted=fit, lower=fit-2*se.fit, upper=fit+2*se.fit)))
pJanka <- cbind(pJanka, prVal)
ry <- with(pJanka, with(janka, range(Hardness, fitted, upper, lower)))
plot(fitted ~ Density, pJanka, ylab="Hardness", type="l",
     ylim=ry, col=grey(0.5))
lines(upper ~ Density, pJanka, lty="dashed", col=grey(0.75))
lines(lower ~ Density, pJanka, lty="dashed", col=grey(0.75))
points(Hardness ~ Density, janka, pch=20, col="red")


###################################################
### code chunk number 18: c17
###################################################
LogM <- lm(log(Hardness) ~ poly(log(Density), 3), janka)
round(summary(LogM)$coef, 4)
LogM1 <- update(LogM, . ~ log(Density))
pJanka <- data.frame(Density = 24:70)  ## just covers actual range
prVal <- with(predict(LogM1, pJanka, type = "resp", se.fit = TRUE),
       exp(cbind(fitted=fit, lower=fit-2*se.fit, upper=fit+2*se.fit)))
pJanka <- cbind(pJanka, prVal)
ry <- with(pJanka, with(janka, range(Hardness, fitted, upper, lower)))
plot(fitted ~ Density, pJanka, ylab="Hardness", type="l",
     ylim=ry, col=grey(0.5))
lines(upper ~ Density, pJanka, lty="dashed", col=grey(0.75))
lines(lower ~ Density, pJanka, lty="dashed", col=grey(0.75))
points(Hardness ~ Density, janka, pch=20, col="red")


###################################################
### code chunk number 19: cexp
###################################################
W <- rexp(10000)
c(mean = mean(W), variance = var(W))


###################################################
### code chunk number 20: c20 (eval = FALSE)
###################################################
## set.seed(1234)
## X <- replicate(500, {
##   tmp <- update(LogM1, weights = rexp(nrow(janka)))
##   predict(tmp, pJanka, type = "resp")  ## predicts in log space
## })
## ci <- t(apply(X, 1, quantile, prob = c(1,39)/40))
## pJanka <- cbind(pJanka, lowerBB = exp(ci[, 1]), upperBB = exp(ci[,2]))
## ry <- with(pJanka, with(janka, range(Hardness, fitted, upper, lower)))
## plot(fitted ~ Density, pJanka, ylab = "Hardness", type = "l",
##      ylim = ry, col=grey(0.5))
## lines(upperBB ~ Density, pJanka, lty = "dashed", col = "green4")
## lines(lowerBB ~ Density, pJanka, lty = "dashed", col = "green4")
## lines(upper ~ Density, pJanka, lty = "dashed", col = grey(0.75))
## lines(lower ~ Density, pJanka, lty = "dashed", col = grey(0.75))
## points(Hardness ~ Density, janka, pch=20, col="red")
## rm(X)


###################################################
### code chunk number 21: c21
###################################################
set.seed(1234)
X <- replicate(500, {
  tmp <- update(LogM1, weights = rexp(nrow(janka)))
  predict(tmp, pJanka, type = "resp")  ## predicts in log space
})
ci <- t(apply(X, 1, quantile, prob = c(1,39)/40))
pJanka <- cbind(pJanka, lowerBB = exp(ci[, 1]), upperBB = exp(ci[,2]))
ry <- with(pJanka, with(janka, range(Hardness, fitted, upper, lower)))
plot(fitted ~ Density, pJanka, ylab = "Hardness", type = "l",
     ylim = ry, col=grey(0.5))
lines(upperBB ~ Density, pJanka, lty = "dashed", col = "green4")
lines(lowerBB ~ Density, pJanka, lty = "dashed", col = "green4")
lines(upper ~ Density, pJanka, lty = "dashed", col = grey(0.75))
lines(lower ~ Density, pJanka, lty = "dashed", col = grey(0.75))
points(Hardness ~ Density, janka, pch=20, col="red")
rm(X)


###################################################
### code chunk number 22: c18 (eval = FALSE)
###################################################
## prVal <- with(predict(mGLM, pJanka, type = "link", se.fit = TRUE),
##               cbind(fittedGLM = fit, lowerGLM = fit-2*se.fit,
##                     upperGLM = fit+2*se.fit)^2)  ## inverse of sqrt
## pJanka <- cbind(pJanka, prVal)
## ry <- with(pJanka, range(ry, fittedGLM, upperGLM, lowerGLM))
## plot(fittedGLM ~ Density, pJanka, ylab = "Hardness", type = "l",
##      ylim = ry, col=grey(0.5), log = "xy", main = "log-log scale")
## axis(2, at = 500*(1:6))
## lines(fitted ~ Density, pJanka, col = "green4")
## lines(upperGLM ~ Density, pJanka, lty = "dashed", col = grey(0.75))
## lines(lowerGLM ~ Density, pJanka, lty = "dashed", col = grey(0.75))
## lines(upper ~ Density, pJanka, lty = "dotted", col = "green4")
## lines(lower ~ Density, pJanka, lty = "dotted", col = "green4")
## points(Hardness ~ Density, janka, pch=20, col="red")


###################################################
### code chunk number 23: c19
###################################################
prVal <- with(predict(mGLM, pJanka, type = "link", se.fit = TRUE),
              cbind(fittedGLM = fit, lowerGLM = fit-2*se.fit,
                    upperGLM = fit+2*se.fit)^2)  ## inverse of sqrt
pJanka <- cbind(pJanka, prVal)
ry <- with(pJanka, range(ry, fittedGLM, upperGLM, lowerGLM))
plot(fittedGLM ~ Density, pJanka, ylab = "Hardness", type = "l",
     ylim = ry, col=grey(0.5), log = "xy", main = "log-log scale")
axis(2, at = 500*(1:6))
lines(fitted ~ Density, pJanka, col = "green4")
lines(upperGLM ~ Density, pJanka, lty = "dashed", col = grey(0.75))
lines(lowerGLM ~ Density, pJanka, lty = "dashed", col = grey(0.75))
lines(upper ~ Density, pJanka, lty = "dotted", col = "green4")
lines(lower ~ Density, pJanka, lty = "dotted", col = "green4")
points(Hardness ~ Density, janka, pch=20, col="red")


###################################################
### code chunk number 24: 02.Rnw:530-532
###################################################
toLatex(sessionInfo())
graphics.off()


