hankinson <- within(read.csv("hankinson.csv"), {
    fTheta <- fTheta/2240
    theta <- base::pi*Angle/180
})

## StartM <- lm(1/fTheta ~ 0+I(cos(theta)^2) + I(sin(theta)^2), hankinson)
## f <- 1/coef(StartM)
## names(f) <- c("f00", "f90")

fStart <- with(hankinson, c(f00 = fTheta[Angle == 00],
                            f90 = fTheta[Angle == 90]))

HankinE <- nls(log(fTheta) ~ log(f00) + log(f90) -
               log(f00*sin(theta)^2 + f90*cos(theta)^2), hankinson,
               start = fStart, trace = TRUE)

## parNames <- c("f00", "f90")
## argNames <- c("theta", parNames)
## Hankinson <- deriv(formula(HankinE)[-2], parNames, argNames)

## HankinED <- nls(log(fTheta) ~ Hankinson(theta, f00, f90), hankinson,
##                 start = fStart, trace = TRUE)

pHankinson <- within(data.frame(Angle = 0:90),
                     theta <- Angle*base::pi/180)

## mle is biased down, hence conservative.
pHankinson$fTheta <- exp(predict(HankinE, pHankinson))

## plot(fTheta ~ Angle, pHankinson, type = "l",
##      xlab = expression(theta*degree),
##      ylab = "Modulus of rupture", log = "y")

nr <- nrow(hankinson)

### update fStart to the LS estimates and turn off tracing
fStart <- structure(coef(HankinE), names = names(fStart))
HankinE <- update(HankinE, trace = FALSE)

Z <- replicate(500, {
  repeat {
    tmpM <- try(update(HankinE, weights = rexp(nr)))
    if(class(tmpM) != "try-error") break
  }
  exp(predict(tmpM, newdata = pHankinson))
})

pHankinson <- within(pHankinson, {
  lims <- apply(Z, 1, quantile, probs = c(1,39)/40)
  lowerBB <- lims[1,]
  upperBB <- lims[2,]
  rm(lims)
})

bsample <- function(dfr) dfr[sample(nrow(dfr), replace = TRUE), ]

Z <- replicate(500, {
  repeat {
    tmpM <- try(update(HankinE, data = bsample(hankinson)))
    if(class(tmpM) != "try-error") break
  }
  exp(predict(tmpM, newdata = pHankinson))
})

pHankinson <- within(pHankinson, {
  lims <- apply(Z, 1, quantile, probs = c(1,39)/40)
  lower <- lims[1,]
  upper <- lims[2,]
  rm(lims)
})

ylim <- with(pHankinson, range(upper, lower, upperBB, lowerBB))

with(pHankinson, {
  par(mar = c(5,5,4,2)+0.1)
  plot(Angle, fTheta, type = "l", ylim = ylim,
       main = "Hankinson's Equation",
       xlab = expression(theta * degree),
       ylab = expression(italic(f)[theta]*":"~~"Modulus of rupture (tons/sq. in.)"),
       axes = FALSE)
  axis(1, at = seq(0, 90, by=15))
  axis(2, at = 1:7, las = 1)
  points(fTheta ~ Angle, hankinson, pch = 20, col = "green3")
  lines(Angle, lower, col="blue", lty = "dotted")
  lines(Angle, upper, col="blue", lty = "dotted")
  lines(Angle, lowerBB, col="red", lty = "dotted")
  lines(Angle, upperBB, col="red", lty = "dotted")
  legend("topright", c("Data", "Model", "Bootstrap limits", "Bayesian Bootstrap"),
         pch = c(20, NA, NA, NA), pt.cex = 1.5,
         lty = c(NA, "solid", "dotted", "dotted"), lwd = 2,
         col = c("green3", "black", "blue", "red"),
         text.col = c("black", "green3", "blue", "red"),
         bty = "n")
  oldPar <- par(usr = c(0,1,0,1))
  text(3/4, 1/2, expression(italic(f)[theta]~~"" == ""~~frac(italic(f)[0] ~ italic(f)[pi/2],
      italic(f)[0]~sin^2~theta~~"" + ""~~italic(f)[pi/2]~cos^2~theta)))
  text(3/4, 2/5, bquote(hat(f)[0]~"" == ""~.(round(coef(HankinE)["f00"], 4))*","~~
                        hat(f)[pi/2]~"" == ""~.(round(coef(HankinE)["f90"], 4))))
  par(oldPar)
})

Store(HankinE, hankinson, pHankinson, bsample)
rm(ylim, fStart, Z, nr)
